using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;
using MeshVR;

// HZMCUACOLLISIONSGLOBAL v1.0
//
// Improve CUA collisions when you need to interact with it on a global level, it should be added ton an empty
// All CUA assets who needs that kind of collisions should have a "root_throwable" parent in their hierarchy to work with it
// This plugins allow to add a "master controller" of collision in the scene and will scan everything at scene load.
// It avoids for the creator to have dozens of plugins loaded on each CUA

namespace HZMCUACollisionsGlobalPlugin
{
    public class HZMCUACollisionsGlobal : MVRScript
    {		
		// Game Objects and components
		private GameObject HZMCUACGO;	
		private GameObject HZMCUACObjectGO;
		private JSONStorableString _HZMCUAInfos;
		
		private List<Atom> enabledAtoms;
		
		private bool HZMCUAG_Done = false;
		
			
		public override void Init()
        {			 
            try
            {
                if (containingAtom.type != "Empty")
                {
                    SuperController.LogError("Please add HZMCUACollisionsGlobal on an Empty Atom");
                    return;
                }
				
				_HZMCUAInfos = new JSONStorableString("Collisions Infos", "");
				UIDynamic stateinfosTextfield = CreateTextField(_HZMCUAInfos, false);
				stateinfosTextfield.height = 1000.0f;
				
				createStaticDescriptionText("Global options squishes","<color=#000><size=32><b>ENABLE PHYSICS</b></size></color>\n<size=28>The button below will enable physics and disable position and rotation on all throwable CUA \"enabled\" (check the list on the left). Place your interactable and throwable assets in the scene then click this button once and save the scene.</size>",true,250);

				UIDynamicButton enablePhysics = CreateButton("Enable physics on all atoms", true);
				if (enablePhysics != null) {
					enablePhysics.button.onClick.AddListener( () => {
						foreach( Atom at in enabledAtoms ) {

							at.GetStorableByID("control").SetBoolParamValue("physicsEnabled", true);
							FreeControllerV3 tmp = at.GetStorableByID("control") as FreeControllerV3;
							tmp.SetPositionStateFromString("Off");
							tmp.currentPositionState = FreeControllerV3.PositionState.Off;
							tmp.SetRotationStateFromString("Off");
							tmp.currentRotationState = FreeControllerV3.RotationState.Off;
						}
					} );
				}
				
				UIDynamicButton disablePhysics = CreateButton("Disable physics on all atoms", true);
				if (disablePhysics != null) {
					disablePhysics.button.onClick.AddListener( () => {
						foreach( Atom at in enabledAtoms ) {

							at.GetStorableByID("control").SetBoolParamValue("physicsEnabled", false);
							FreeControllerV3 tmp = at.GetStorableByID("control") as FreeControllerV3;
							tmp.SetPositionStateFromString("On");
							tmp.currentPositionState = FreeControllerV3.PositionState.On;
							tmp.SetRotationStateFromString("On");
							tmp.currentRotationState = FreeControllerV3.RotationState.On;
						}
					} );
				}
			}
            catch(Exception e)
            {
                SuperController.LogError("HZMCUACollisionsGlobal - Exception caught: " + e);
            }
		}
		
		void Start()
        {
			try
            {

			}
            catch(Exception e)
            {
                SuperController.LogError("HZMCUACollisionsGlobal - Exception caught: " + e);
            }
        }
		
		void Update() {
			// If every CUA atom has been scanned, then no more stuff to do in the update
			if( HZMCUAG_Done == true ) return;
			// Waiting for the loading to finish, then applying the detection mode on CUA that have an asset with the root_throwable inside
			if( HZMCUAG_Done == false && !SuperController.singleton.isLoading ) {
				try
				{
					
					enabledAtoms = new List<Atom>();
					String tmpTitle = "<b>GLOBAL COLLISION INFOS: </b>\n"
									  + "Below is the list of CUA with continuous collisions enabled or ignored.\n";
									  
					String tmpEnabled = "\n<b>CUA enabled</b>\n";
					String tmpIgnored = "\n<b>CUA ignored</b>\n";
					foreach(Atom at in SuperController.singleton.GetAtoms() ) {
						if( at.type == "CustomUnityAsset" ) {
							Transform atTR = getCUAObject( at.reParentObject );
							Transform atChildTr = getChildRoot( atTR, "root_throwable" );
							if( atChildTr != null ) {
								Rigidbody RB = atTR.gameObject.GetComponent<Rigidbody>();
								RB.collisionDetectionMode = CollisionDetectionMode.ContinuousDynamic;
								tmpEnabled += at.name + "\n";
								enabledAtoms.Add(at);
							} else {
								tmpIgnored += at.name + "\n";
							}
							
							
							_HZMCUAInfos.val = tmpTitle + tmpEnabled + tmpIgnored;
						}
					}
					
					HZMCUAG_Done = true;
				}
				catch(Exception e)
				{
					SuperController.LogError("HZMCUACollisionsGlobal - Exception caught while initializing: " + e);
				}
			}
		}
			
		// **************************
		// Callbacks & similar
		// **************************
		
			
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private static void disableScrollOnText(UIDynamicTextField target) {
			ScrollRect targetSR = target.UItext.transform.parent.transform.parent.transform.parent.GetComponent<ScrollRect>();
			if( targetSR != null ) {
				targetSR.horizontal = false;
				targetSR.vertical = false;
			}
		}
		
		private JSONStorableString createStaticDescriptionText( string DescTitle, string DescText, bool rightSide, int fieldHeight ) {
			JSONStorableString staticDescString = new JSONStorableString(DescTitle,DescText);
			UIDynamicTextField staticDescStringField = CreateTextField(staticDescString, rightSide);
			staticDescStringField.backgroundColor = new Color(1f, 1f, 1f, 0f);
			LayoutElement sdsfLayout = staticDescStringField.GetComponent<LayoutElement>();
			sdsfLayout.preferredHeight = sdsfLayout.minHeight = fieldHeight;
			staticDescStringField.height = fieldHeight;
			disableScrollOnText(staticDescStringField);

			return staticDescString;
		}
		
        // Recursive function searching for a transform
        private Transform getCUAObject( Transform parent ) {
            foreach( Transform child in parent )
            {
				if( child.name == "object" ) {
					return child;
				} else {
					Transform childSearch = getCUAObject(child);
					if (childSearch != null) return childSearch;
				}
            }
        
            return null;
        }
		
		private Transform getChildRoot( Transform parent, string searchName ) {
			foreach( Transform child in parent )
			{
				if( child.name == searchName ) {
					return child;
				} else {
					Transform childSearch = getChildRoot(child,searchName);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		// **************************
		// Time to cleanup !
		// **************************
		void OnDestroy() {
			// Cleaning modified native VAM assets

		}

	}
}